package dev.thinhha.tunnel_server.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;

@Configuration
@EnableWebSocket
public class WebsocketConfig implements WebSocketConfigurer {
    private final WebSocketHandler webSocketHandler;
    private final FacadeWebSocketHandler facadeWebSocketHandler;
    private final WebSocketTunnelHandler webSocketTunnelHandler;
    private final JwtWebSocketHandshakeInterceptor jwtHandshakeInterceptor;

    public WebsocketConfig(WebSocketHandler webSocketHandler, 
                          FacadeWebSocketHandler facadeWebSocketHandler,
                          WebSocketTunnelHandler webSocketTunnelHandler,
                          JwtWebSocketHandshakeInterceptor jwtHandshakeInterceptor) {
        this.webSocketHandler = webSocketHandler;
        this.facadeWebSocketHandler = facadeWebSocketHandler;
        this.webSocketTunnelHandler = webSocketTunnelHandler;
        this.jwtHandshakeInterceptor = jwtHandshakeInterceptor;
    }

    @Override
    public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
        registry.addHandler(webSocketHandler, "/client")
                .addInterceptors(jwtHandshakeInterceptor)
                .setAllowedOriginPatterns("*");
        
        registry.addHandler(webSocketTunnelHandler, "/ws/**")  // WebSocket tunnel endpoints
                .setAllowedOriginPatterns("*");
        
        registry.addHandler(facadeWebSocketHandler, "*")
                .setAllowedOriginPatterns("*");
    }
}
