package dev.thinhha.tunnel_server.controller;

import dev.thinhha.tunnel_server.dto.TunnelRequestDto;
import dev.thinhha.tunnel_server.dto.TunnelResponseDto;
import dev.thinhha.tunnel_server.service.TunnelService;
import dev.thinhha.tunnel_server.types.HttpMethod;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/tunnel")
@RequiredArgsConstructor
public class TunnelController {

    private final TunnelService tunnelService;

    @RequestMapping(value = "/{clientId}/**", method = {RequestMethod.GET, RequestMethod.POST, RequestMethod.PUT, RequestMethod.DELETE, RequestMethod.PATCH, RequestMethod.HEAD, RequestMethod.OPTIONS})
    public ResponseEntity<?> tunnelRequest(@PathVariable String clientId, HttpServletRequest request) throws IOException {
        String requestId = UUID.randomUUID().toString();
        
        // Extract path after /tunnel/{clientId}
        String fullPath = request.getRequestURI();
        String tunnelPath = fullPath.substring(("/tunnel/" + clientId).length());
        
        // Create tunnel request DTO
        TunnelRequestDto tunnelRequest = new TunnelRequestDto();
        tunnelRequest.setRequestId(requestId);
        tunnelRequest.setMethod(HttpMethod.valueOf(request.getMethod()));
        tunnelRequest.setPath(tunnelPath);
        tunnelRequest.setHeaders(tunnelService.extractHeaders(request));
        tunnelRequest.setBody(request.getInputStream().readAllBytes());
        tunnelRequest.setClientShortName(clientId);
        
        try {
            // Send request to WebSocket client and wait for response
            CompletableFuture<TunnelResponseDto> responseFuture = tunnelService.sendTunnelRequest(tunnelRequest);
            TunnelResponseDto response = responseFuture.get(30, TimeUnit.SECONDS);
            
            return ResponseEntity.status(response.getStatusCode())
                    .headers(httpHeaders -> response.getHeaders().forEach(httpHeaders::add))
                    .body(response.getBody());
                    
        } catch (Exception e) {
            return ResponseEntity.status(502).body("Tunnel request failed: " + e.getMessage());
        }
    }
}
